/*
 * PSP Software Development Kit - http://www.pspdev.org
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPSDK root for details.
 *
 * main.cpp - Adhoc support for LuaPlayer using the PSPSDK.
 */
#include <stdlib.h>
#include <pspkernel.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>

#include <pspumd.h>
#include <pspctrl.h>
#include <pspsdk.h>
#include <psputility.h>
#include <pspusb.h>
#include <pspusbstor.h>
#include <psppower.h>
#include <pspdebug.h>
#include <unistd.h>
#include <sys/stat.h>
#include <pspnet.h>
#include <pspnet_adhoc.h>
#include <pspnet_adhocctl.h>
#include <pspnet_adhocmatching.h>
#include <pspwlan.h>

extern "C" {

#include "luamodule.h"
#include "../../src/graphics.h"

int init(lua_State *L );

}  // extern "C"

PSP_MODULE_INFO("ADHOC", 0, 0, 1);
PSP_HEAP_SIZE_KB(240);
PSP_NO_CREATE_MAIN_THREAD(); 

int pdpId = 0;
int matchingId = 0;
char g_mac[6];
int g_unk1=0;
int g_matchEvent=0;
int g_matchOptLen=0;
char g_matchOptData[1000];
int matchChanged = 0;


static bool adhocInitialized = false;

void matchingCallback(int unk1, int event, char *mac2, int optLen, char *optData)
{
	if(event==MATCHING_JOINED)
	{
		//AddPsp(mac2, optData, optLen); -- not implemented
	}
	else if(event==MATCHING_DISCONNECT)
	{
		//DelPsp(mac2); -- not implemented
	}
	else
	{
		g_unk1 = unk1;
		g_matchEvent = event;
		g_matchOptLen = optLen;
		strncpy(g_matchOptData, optData, optLen);
		memcpy(g_mac, mac2, sizeof(char) * 6);
		matchChanged = 1;
	}
}

static int adhoc_init(lua_State* L)
{
	if (lua_gettop(L) != 1) return luaL_error(L, "Adhoc.init(id) expects a string of nine characters as a game id");
	if (adhocInitialized) return 0;
	const char *tempproductid = luaL_checkstring(L, 1);
	char mac[6];
	u32 err;

	err = sceNetInit(0x20000, 0x20, 0x1000, 0x20, 0x1000);
    if (err != 0) return luaL_error(L, "sceNetInit failed.");

	err = sceNetAdhocInit();
	if (err != 0) return luaL_error(L, "sceNetAdhocInit failed.");

	struct productStruct productS;
	productS.unknown = 0;
	strcat(productS.product, tempproductid);
	err = sceNetAdhocctlInit(0x2000, 0x30, &productStruct );
	if (err != 0) return luaL_error(L, "sceNetAdhocctlInit failed.");
	
	err = sceNetAdhocctlConnect(0x00);
	if (err != 0) return luaL_error(L, "sceNetAdhocctlConnect failed.");

    while (1)
    {
        int state;
        err = sceNetAdhocctlGetState(&state);
        if (err != 0)
        {
			sceKernelDelayThread(10*1000000); // 10sec to read before exit
			return -1;
        }
        if (state == 1) break;  // connected

        // wait a little before polling again
        sceKernelDelayThread(50*1000); // 50ms
    }
	
	sceWlanGetEtherAddr(mac);

	pdpId = sceNetAdhocPdpCreate(mac, 0x309, 0x400, 0);
	if(pdpId <= 0) return luaL_error(L, "sceNetAdhocPdpCreate failed.");

	err = sceNetAdhocMatchingInit(0x20000);
	if(err != 0) return luaL_error(L, "sceNetAdhocMatchingInit failed");

	matchingId = sceNetAdhocMatchingCreate( 3, 0xa, 0x22b, 0x800, 0x2dc6c0, 0x5b8d80, 3, 0x7a120, matchingCallback);

	char tempStr[100];
	tempStr[0] = '\0';

	printf("sceNetAdhocMatchingStart()\n");
	err = sceNetAdhocMatchingStart(matchingId, 0x10, 0x2000, 0x10, 0x2000, 6, "adhoc\0");	
	if(err != 0) return luaL_error(L, "sceNetAdhocMatchingStart failed.");

	adhocInitialized = true;
	return 0;
}

static int adhoc_term(lua_State* L)
{
	if (lua_gettop(L) != 0) return luaL_error(L, "no arguments expected.");
	if (!adhocInitialized) return 0;
    u32 err;
	
	err = sceNetAdhocctlDisconnect();
	if(err != 0) return luaL_error(L, "sceNetAdhocctlDisconnect failed.");
		
	err = sceNetAdhocPdpDelete(pdpId,0);
	if(err != 0) return luaL_error(L, "sceNetAdhocPdpDelete failed.");
		
	err = sceNetAdhocMatchingStop(matchingId);
	if(err != 0) return luaL_error(L, "sceNetAdhocMatchingStop failed.");
		
	err = sceNetAdhocMatchingDelete(matchingId);
	if(err != 0) return luaL_error(L, "sceNetAdhocMatchingDelete failed.");
		
	err = sceNetAdhocMatchingTerm();
	if(err != 0) return luaL_error(L, "sceNetAdhocMatchingTerm failed.");
		
	err = sceNetAdhocctlTerm();
	if(err != 0) return luaL_error(L, "sceNetAdhocctlTerm failed.");

	err = sceNetAdhocTerm();
	if(err != 0) return luaL_error(L, "sceNetAdhocTerm failed.");

	err = sceNetTerm();
	if(err != 0) return luaL_error(L, "sceNetTerm failed.");

	adhocInitialized = false;
	return 0;
}

static const luaL_reg Adhoc_functions[] = {
  {"init",	adhoc_init},
  {"term",	adhoc_term},
  {0,0}
};

int init(lua_State *L )
{
	luaL_openlib(L, "Adhoc", Adhoc_functions, 0);
	return 0;
}

int main(int argc, char *argv[] )
{
	return 0;
}

